//+------------------------------------------------------------------+
//|                                                     GridGale.mq4 |
//|                                      Coded By Santhosh44, Elance |
//+------------------------------------------------------------------+
#property copyright ""
#property link      ""
#define   NL        "\n"

extern string  s1 = "==== Trigger Management ====";
extern int     Magic          = 555;
extern double  MaxSpread      = 4.9;
extern string  TriggerGrid    = "50;25;50;50;100;100";
extern bool    RSIMode        = true;
extern bool    RSIClose       = true;
extern string  RSIPeriod      = "5;7;9;14;14;14;9;5";
extern string  RSIOverBought  = "55;60;60;60;65;70";
extern string  RSIOverSold    = "45;40;40;40;35;30";
extern string  RearViewMirror = "5;0;0;1;1;2;3;4;5;10;12";

extern string  s2 = "==== Order Management ====";
extern string  LotSize        = "0.01;0.02;0.03;0.05;0.07";
extern string  UltimateTP     = "4;3;4;5;5;5;5;6;7;3;4";
extern string  TradeCount     = "2;2;2;2;2;3;4;4;4;4;1;1";
extern string  GridSpacing    = "50;50;50;50;50;50;100;100;100";
extern string  TimeOut        = "0;0;0;1;1;1;1;2;3;4;0;0;0;6;0;60";
extern string  StopLoss       = "45;45;45;45;45;45;45;45;100;100;100";

extern string  s3 = "==== Money Management ====";
extern double  MaxEquityStop   = 0;
extern double  MaxBalanceStop  = 0;
extern double  MinEquityStop   = 0;
extern double  MinBalanceStop  = 0;

extern string  s4 = "==== Other settings ====";
extern int     FontSize                = 8;
extern color   FontColor               = Gold;
extern int     ColumnSpacing           = 50;
extern int     TableCorner             = 0;
extern int     NumTriggerGridLines     = 1 ;

double         myPoint, divider;
int            cLevel;
datetime       cLevelTime, timeOutStart;
string         msg, tablemsg, errormsg;
int            triggergrid[100], rview[100], rsiperiod[100], rsiob[100], rsios[100];
int            tgcount, rvcount, rscount, obcount, oscount;
double         lotsize[100];
int            tprofit[100],tradecnt[100],gridspace[100],timeout[100],sloss[100];
int            locount, tpcount, trcount, gscount, ticount, slcount;
double         ugrid[4], lgrid[4];
double         mmtable[100][2];
bool           waitfornext, eastopped;

int            Slippage          = 5;
int            NumRetries        = 10;
int            RetryDelayInSec   = 5; 
string         TradeComment      = "";

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   
   if(Digits==3 || Digits==5) {
      myPoint = 10 * Point;
      divider = 10;
   }  else {
      myPoint = Point;
      divider = 1;
   }
   
   waitfornext = true ;
   
   tgcount = StrToIntegerArray(TriggerGrid, triggergrid); 
   rvcount = StrToIntegerArray(RearViewMirror, rview); 
   rscount = StrToIntegerArray(RSIPeriod, rsiperiod);
   obcount = StrToIntegerArray(RSIOverBought, rsiob);
   oscount = StrToIntegerArray(RSIOverSold, rsios);
   locount = StrToDoubleArray(LotSize,lotsize);
   trcount = StrToIntegerArray(TradeCount, tradecnt); 
   gscount = StrToIntegerArray(GridSpacing, gridspace); 
   tpcount = StrToIntegerArray(UltimateTP, tprofit);
   slcount = StrToIntegerArray(StopLoss, sloss);
   ticount = StrToIntegerArray(TimeOut, timeout);
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   ResetGridLines();
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
//----
   if(eastopped) {
      Comment(errormsg);
      return(0);
   }
   
   if(CountOrders() == 0) {
      if(waitfornext) {
         DeletePendingOrders();
         
         if((MinBalanceStop > 0 && AccountBalance() <= MinBalanceStop) || (MaxBalanceStop > 0 && AccountBalance() >= MaxBalanceStop)) {
            Comment("Account balance not within the limit set. EA disabled");
            return(0);
         }
         
         cLevel = MoneyManagement();

         if(cLevel == -1) {
            errormsg = "ERROR ! Money Management module failed. EA disabled";
            eastopped = true;
            return(0);
         } else if(!ValidateInputs(cLevel)) {
            eastopped =  true;
            return(0);
         }
         
         if(cLevel == 0)
            ArrayInitialize(mmtable,EMPTY_VALUE);
         
         waitfornext  = false;
         cLevelTime   = 0;
         timeOutStart = 0;
      }

      if(MarketInfo(Symbol(), MODE_SPREAD)/divider <= MaxSpread)
         StartNewSequence();
      else
         msg = "New entries not allowed. Current spread ("+ (MarketInfo(Symbol(), MODE_SPREAD)/divider) + ") greater than MaxSpread"+NL;
   } 
   else {
      ManageSequence();
   }
   
   DisplayTable() ;
   msg = "Current Sequence : "+(cLevel+1)+NL; 
   if(!waitfornext)
      msg = msg + "Waiting for trigger..."+NL+NL;
   else 
      msg = msg + CountOrders()+" trades opened..."+NL+NL;
      
   Comment(msg+tablemsg);
//----
   return(0);
  }
//+------------------------------------------------------------------+

bool ValidateInputs(int level)
{
   if(level >= tgcount) {
      errormsg = "TriggerGrid input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   } else if(level >= rvcount) {
      errormsg = "RearViewMirror input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   } else if(RSIMode && level >= rscount) {
      errormsg = "RSIPeriod input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   } else if(RSIMode && level >= obcount) {
      errormsg = "RSIOverBought input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   } else if(RSIMode && level >= oscount) {
      errormsg = "RSIOverSold input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   } else if(level >= locount) {
      errormsg = "LotSize input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   } else if(level >= trcount) {
      errormsg = "TradeCount input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   } else if(level >= gscount) {
      errormsg = "GridSpacing input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   } else if(level >= tpcount) {
      errormsg = "UltimateTP input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   } else if(level >= slcount) {
      errormsg = "StopLoss input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   } else if(level >= ticount) {
      errormsg = "TimeOut input not specified for Sequence "+(level+1)+". EA disabled";
      return(false);
   }

   return(true);      
}

int MoneyManagement()
{
   Print("Money Management called...");
   if(cLevelTime == 0) {
      return(0);
   }
      
   double pl = GetSequencePL();
   Print("Sequence P/L:"+pl);
   int nextstep ;
   
   int index = GetEmptyColumn();
   //Print("index:"+index);
   if(index < 0)
      return(-1);
   
   mmtable[index][0] = pl;
   if(index == 0)
      mmtable[index][1] = pl;
   else {
      mmtable[index][1] = mmtable[index-1][1] + pl;
      ReorderTable(index);
   }
   
   if(mmtable[0][1] > 0) {
      cLevelTime = 0;
      nextstep = 0 ;
   } else {
      nextstep = GetEmptyColumn();
   }
   
   Print("nextstep:"+nextstep);
   
   return(nextstep);
}

void DisplayTable()
{
   PrintLN("tablerow0","Sequence#",40,3, "Verdana Bold");
   PrintLN("tablerow1","Loss",60,3, "Verdana Bold");
   PrintLN("tablerow2","Running Total",80,3, "Verdana Bold");
   
   //Print("DisplayTable called. cLevel = "+cLevel);
   for(int i = 0 ; i <= cLevel ; i++) {
      if(i == 0)
         int x = 125;
      else
         x = 125 + ColumnSpacing*i;
         
      PrintLN("tablerow0"+i,(i+1),40,x,"Verdana Bold");
      
      if(mmtable[i][1] == EMPTY_VALUE) {
         string pl = "*";
         string running = "*";
      }
      else {
         pl = DoubleToStr(mmtable[i][0],1);
         running = DoubleToStr(mmtable[i][1],1);
      }
      PrintLN("tablerow1"+i,pl,60,x);
      PrintLN("tablerow2"+i,running,80,x);
      
      if(mmtable[i][1] == EMPTY_VALUE) {
         ClearTable(i+1);
         break;
      }
   }
}

void ClearTable(int start)
{
   for(int i = start ; i < 100 ; i++) {
      if(ObjectFind("tablerow0"+start) >= 0) {
         ObjectDelete("tablerow0"+start);
         ObjectDelete("tablerow1"+start);
         ObjectDelete("tablerow2"+start);
      }
   }
}

void ReorderTable(int index)
{
   while(index > 0) {
      Print("previous : "+mmtable[index-1][1]+" current : "+mmtable[index][1]);
      if(mmtable[index][1] > mmtable[index-1][1]) {
         Print("shifting running total to "+(index-1));
         mmtable[index-1][0] = mmtable[index][0];
         mmtable[index-1][1] = mmtable[index][1];
         mmtable[index][1] = EMPTY_VALUE;
         index--;
      } else
         break;
   }
}

int GetEmptyColumn()
{
   for(int i = 0 ; i < 100 ; i++)
      if(mmtable[i][1] == EMPTY_VALUE)
         return(i);
   
   return(-1);
}

double GetSequencePL()
{
   double pl;
   
   for (int cnt = OrdersHistoryTotal() - 1; cnt >= 0; cnt --)
   {
      OrderSelect (cnt, SELECT_BY_POS, MODE_HISTORY);
      if (OrderType() <= OP_SELL && OrderSymbol() == Symbol() && OrderMagicNumber()==Magic ) {  
         if(OrderOpenTime() >= cLevelTime)
            //Print("orderprofit:"+OrderProfit());
            pl = pl + OrderProfit();
      }
   } 
   
   //Print("pl:"+pl);
   return(pl);
}

void StartNewSequence()
{
   msg = "Current Sequence : "+ (cLevel+1)+" Waiting for first trigger.. "+NL;
   if(ugrid[0] == 0)
      PlotGridLines();
   
   if(Bid >= ugrid[0]){
      double entry = ugrid[0];
      ResetGridLines();
      Print("Confirm Buy trade..");
      if(RearViewCheck(entry) && RSICheck(OP_BUY))
         OrderInitialize(OP_BUY);
      else
         PlotGridLines();
   } else if(Bid <= lgrid[0]) {
      entry = lgrid[0];
      ResetGridLines();
      Print("Confirm Sell trade..");
      if(RearViewCheck(entry) && RSICheck(OP_SELL))
         OrderInitialize(OP_SELL);
      else
         PlotGridLines();
   }
}

bool RearViewCheck(double level)
{
   double barcount = rview[cLevel];
   //Print("barcount:"+barcount+"level:"+level);
   for(int i = 1 ; i <= barcount ; i++) {
      //Print("Low:"+Low[i]+" High:"+High[i]);
      if(Low[i] <= level && High[i] >= level) {
         //Print("Rearview check failed");
         return(false);
      }
   }
   
   //Print("Rearview check success");
   return(true);
}

bool RSICheck(int type)
{
   if(!RSIMode)
      return(true);
      
   double period  = rsiperiod[cLevel];
   double ob      = rsiob[cLevel];
   double os      = rsios[cLevel];
   
   double rsi = iRSI(NULL, 0, period, PRICE_CLOSE, 0);
   //Print("RSI ="+rsi);
   if((type == OP_BUY && rsi >= ob) || (type == OP_SELL && rsi <= os)) {
      //Print("RSI check success");
      return(true);
   }
   
   //Print("RSI check failed");  
   return(false);
}

void OrderInitialize(int type)
{
   double lot = lotsize[cLevel];
   int ticket = PlaceOrder(type, lot);
   if(ticket > 0) {
      waitfornext = true;
      OrderSelect(ticket, SELECT_BY_TICKET);
      cLevelTime = OrderOpenTime();
      timeOutStart = Time[0];
      PlaceGridTrades(OrderOpenPrice(), lot, OrderType()+4, OrderStopLoss(), OrderTakeProfit());
   }
}

void PlaceGridTrades(double entry, double lot, int type, double sl, double tp)
{
   double count = tradecnt[cLevel];
   double space = gridspace[cLevel];
   
   Print("count:"+count+" space:"+space);
   double mult = 1;
   if (type == OP_SELLSTOP)
      mult = -1;
   
   for(int i = 1; i <= count ; i++) {
      double pendentry = entry + mult * i * space * myPoint;
      PlaceOrder(type, lot, pendentry, sl, tp);
   }
}

double PlaceOrder(int type, double lot, double entry = 0, double sl = 0, double tp = 0)
{
   Print("PlaceOrder called.. Type : "+TypeToStr(type));
   color  col = CLR_NONE;        
      
   int i = 0;
   while(i < NumRetries) {
      i += 1;
      while(IsTradeContextBusy())Sleep(RetryDelayInSec*1000);
	   RefreshRates();
	   if(type == OP_BUY) {
         entry = Ask;
         col   = Blue;
      }
      else if (type == OP_SELL) {
         entry = Bid;
         col   = Red;
      }

      Print("Try "+i+" : Open "+TypeToStr(type)+" order. Entry : "+DoubleToStr(entry,Digits));
      int ticket = OrderSend(Symbol(),type,lot,entry,Slippage,0,0,TradeComment,Magic,0,col);
      if(ticket <= 0){
         Print("ERROR opening order. ErrorCode:"+GetLastError());
         if(i == NumRetries) {
            Print("*** Final retry to OPEN ORDER failed ***");
            return(false);
         }
      } else {
         SetSLandTP(ticket, 0, tp);
         i = NumRetries;
         return(ticket);
      }
   }
   
   return(0);
}

void SetSLandTP(int tick, double sl, double tp)
{
   double gridtp = tprofit[cLevel]; 
   double pipssl = sloss[cLevel];
   double space = gridspace[cLevel];

   OrderSelect(tick,SELECT_BY_TICKET);   
   if(OrderType() <= OP_SELL) {
      if(OrderType() == OP_BUY) {
         tp = OrderOpenPrice() + gridtp * space * myPoint;
         sl = OrderOpenPrice() - pipssl * myPoint;
      }
      else if(OrderType() == OP_SELL) {
         tp = OrderOpenPrice() - gridtp * space * myPoint;
         sl = OrderOpenPrice() + pipssl * myPoint;
      }
   }      
   
   int i = 0;
   while(i < NumRetries) {
      i += 1;
      while(IsTradeContextBusy())Sleep(1000);
	   RefreshRates();
      int t = OrderModify(tick, OrderOpenPrice(), NormalizeDouble(sl,Digits), NormalizeDouble(tp, Digits), 0);
      if(t <= 0) {
          Print("*** ERROR modifying market order. Unable to set SL/TP. SL: "+NormalizeDouble(sl,Digits)+" TP: "+NormalizeDouble(tp,Digits)+" ErrorCode: "+GetLastError());
          if(i == NumRetries) {
            Print("*** Final retry to SET SL/TP failed. Manually set the same or close the open order ***");
            return(0);
         }
      } else {
         i = NumRetries;
      }
   }
}

void ResetGridLines()
{
   ObjectsDeleteAll();
   for(int i = 0 ; i < NumTriggerGridLines ; i++) {
      ugrid[i] = 0;
      lgrid[i] = 0;
   }
}

void PlotGridLines()
{
   double grid    = triggergrid[cLevel];
   double upper = NormalizeDouble(Bid/myPoint,0);
   int level = 0 ;
   while(level <= grid) {
      upper = upper + 1;
      if(MathMod(upper, grid) == 0)
         break;
      level++;
   }
   
   double bstart = upper*myPoint;
   double sstart = bstart - grid*myPoint;
   for(int i = 0 ; i < NumTriggerGridLines ; i++) {
      ugrid[i] = NormalizeDouble(bstart + i*grid*myPoint,Digits) ;
      lgrid[i] = NormalizeDouble(sstart - i*grid*myPoint,Digits) ;
      DrawTrendLine("Buy"+i,ugrid[i],Time[0],ugrid[i],Time[0]+86400,DodgerBlue);
      DrawTrendLine("Sell"+i,lgrid[i],Time[0],lgrid[i],Time[0]+86400,Magenta);
   }
   
}

void DrawTrendLine(string name, double y1, datetime x1, double y2, datetime x2, color lineColor)
{
   ObjectDelete(name);
   ObjectCreate(name, OBJ_TREND, 0, x1, y1, x2, y2);
   ObjectSetText(name, DoubleToStr(y1,Digits));   
   ObjectSet(name, OBJPROP_RAY, 1);
   ObjectSet(name, OBJPROP_COLOR, lineColor);
   ObjectSet(name, OBJPROP_STYLE, STYLE_DOT);
}


void ManageSequence()
{
   UpdateSL();
   ManageTimeout();
   if(RSIMode && RSIClose)
      RSIExit();
   EquityCheck();
}

void RSIExit()
{
   double period  = rsiperiod[cLevel];
   double ob      = rsiob[cLevel];
   double os      = rsios[cLevel];

   for (int y = 0; y < OrdersTotal(); y++)
   {
      OrderSelect (y, SELECT_BY_POS, MODE_TRADES);
      if(OrderType() <= OP_SELL && OrderSymbol() == Symbol() && OrderMagicNumber() == Magic) {
         if(iBarShift(NULL, 0, OrderOpenTime()) >= 1) {
            double rsi = iRSI(NULL, 0, period, PRICE_CLOSE, 0);
            if((OrderType() == OP_BUY && rsi < ob) || (OrderType() == OP_SELL && rsi > os)) {
               Print("RSI = "+rsi+" Close all open positions.");
               CloseAllPositions();
            }
         }
      }
      
   }
}

void EquityCheck()
{
   bool closeall ;
   if(MaxEquityStop > 0 && AccountEquity() >= MaxEquityStop) {
      Print("Account Equity >= MaxEquityStop . Close all positions.");
      closeall = true;
   } else if(MinEquityStop > 0 && AccountEquity() <= MinEquityStop) {
      Print("Account Equity <= MinEquityStop . Close all positions.");
      closeall = true;
   } else if(MaxBalanceStop > 0 && AccountBalance() >= MaxBalanceStop) {
      Print("Account Balance >= MaxBalanceStop . Close all positions.");
      closeall = true;
   } else if(MinBalanceStop > 0 && AccountBalance() <= MinBalanceStop) {
      Print("Account Equity <= MinBalanceStop . Close all positions.");
      closeall = true;
   }
   
   if(closeall) {
      CloseAllPositions();
      cLevelTime   = 0;
   }
}

void ManageTimeout()
{
   double tout    = timeout[cLevel];
   if(tout == 0 || timeOutStart == 0)
      return(0);
      
   if(iBarShift(NULL,0,timeOutStart) >= tout) {
      if(AllTradesInProfit()) {
         Print("Execute Timeout Closeall..");
         CloseAllPositions();
      }
      else {
         Print("Timeout deactivated");
         timeOutStart = 0;
      }
   }
}

bool AllTradesInProfit()
{
   for (int y = 0; y < OrdersTotal(); y++)
   {
      OrderSelect (y, SELECT_BY_POS, MODE_TRADES);
      if(OrderType() <= OP_SELL && OrderSymbol() == Symbol() && OrderMagicNumber() == Magic)
         if(OrderProfit() <= 0)
            return(false);
   }
   
   return(true);
}

void UpdateSL()
{
   bool updatesl = false;
   
   for (int y = 0; y < OrdersTotal(); y++)
   {
      OrderSelect (y, SELECT_BY_POS, MODE_TRADES);
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic) {
         if(OrderType() <= OP_SELL && OrderStopLoss() == 0) {
            timeOutStart = Time[0];
            double pipssl = sloss[cLevel];
            if(OrderType() == OP_BUY)
               double newsl  = OrderOpenPrice() - pipssl * myPoint;
            else
               newsl = OrderOpenPrice() + pipssl * myPoint;
            updatesl = true;
            break;
         }
      }
   }
   
   if(updatesl) {
      UpdateSLForAll(newsl);
   }
}

void UpdateSLForAll(double newsl)
{
   for (int y = 0; y < OrdersTotal(); y++)
   {
      OrderSelect (y, SELECT_BY_POS, MODE_TRADES);
      if (OrderType() <= OP_SELL && OrderSymbol() == Symbol() && OrderMagicNumber() == Magic) {
         int i = 0;
         while(i < NumRetries) {
            i += 1;
            while(IsTradeContextBusy())Sleep(1000);
	         RefreshRates();
            int t = OrderModify(OrderTicket(), OrderOpenPrice(), NormalizeDouble(newsl,Digits), OrderTakeProfit(), 0);
            if(t <= 0) {
                Print("*** ERROR modifying order. Unable to modify SL. SL: "+NormalizeDouble(newsl,Digits)+" ErrorCode: "+GetLastError());
                if(i == NumRetries) {
                  Print("*** Final retry to SET SL/TP failed. Manually set the same or close the open order ***");
                  return(0);
               }
            } else {
               i = NumRetries;
            }
         }
      }
   }
}

int CountOrders()
{
   int count = 0;
      
   for (int y = 0; y < OrdersTotal(); y++)
   {
      OrderSelect (y, SELECT_BY_POS, MODE_TRADES);
      if (OrderType() <= OP_SELL && OrderSymbol() == Symbol() && OrderMagicNumber() == Magic)
         count++;
   }
   
   return(count);
}

int StrToDoubleArray(string str, double &a[], string delim=";", int init=0)
{
	for (int i=0; i<ArraySize(a); i++)
		a[i] = init;
	int z1=-1, z2=0;
	if (StringRight(str,1) != delim)  str = str + delim;
	for (i=0; i<ArraySize(a); i++)  {
		z2 = StringFind(str,delim,z1+1);
		a[i] = StrToNumber(StringSubstr(str,z1+1,z2-z1-1));
		if (z2 >= StringLen(str)-1)   break;
		z1 = z2;
	}
	return(StringFindCount(str,delim));
}

int StrToIntegerArray(string str, int &a[], string delim=";", int init=0)
{
	for (int i=0; i<ArraySize(a); i++) 
		a[i] = init;
	int z1=-1, z2=0;
	if (StringRight(str,1) != delim)  str = str + delim;
	for (i=0; i<ArraySize(a); i++)  {
		z2 = StringFind(str,delim,z1+1);
		a[i] = StrToNumber(StringSubstr(str,z1+1,z2-z1-1));
		if (z2 >= StringLen(str)-1)   break;
		z1 = z2;
	}
	return(StringFindCount(str,delim));
}

string StringRight(string str, int n=1)
{
	if (n > 0)  return(StringSubstr(str,StringLen(str)-n,n));
	if (n < 0)  return(StringSubstr(str,-n,StringLen(str)-n));
	return("");
}

double StrToNumber(string str)
{
	int    dp   = -1;
	int    sgn  = 1;
	double num  = 0.0;
	for (int i=0; i<StringLen(str); i++)  {
		string s = StringSubstr(str,i,1);
		if (s == "-")  sgn = -sgn;   else
			if (s == ".")  dp = 0;       else
			if (s >= "0" && s <= "9")  {
				if (dp >= 0)  dp++;
				if (dp > 0)
					num = num + StrToInteger(s) / MathPow(10,dp);
				else
					num = num * 10 + StrToInteger(s);
			}
	}
	return(num*sgn);
}

int StringFindCount(string str, string str2)
{
	int c = 0;
	for (int i=0; i<StringLen(str); i++)
	if (StringSubstr(str,i,StringLen(str2)) == str2)  c++;
	return(c);
}

string TypeToStr(int type)
{
   if(type == OP_BUY)
      return("BUY");
   else if(type == OP_BUY)
      return("SELL");
   else if(type == 4)
      return("BUY STOP");
   else if(type == 5)
      return("SELL STOP");
}

void DeletePendingOrders()
{
    bool success;
    for (int cnt = OrdersTotal() - 1; cnt >= 0; cnt --){
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber() == Magic && OrderSymbol() == Symbol() && OrderType() > OP_SELL) {
         int i = 0;
         while(i < NumRetries) {
            i += 1;
            while(IsTradeContextBusy())Sleep(RetryDelayInSec*1000);
      	   RefreshRates();
            
            Print("Try "+i+" : Delete "+OrderTicket());
            success = OrderDelete(OrderTicket());
            if(!success){
               Print("Failed to delete pending order "+OrderTicket()+" Error code:"+GetLastError());
               if(i == NumRetries)
                  Print("*** Final retry to DELETE PENDING ORDER failed. Delete the trade manually ***");
            } else
               i = NumRetries;
         }
      }
    } 
}

bool CloseAllPositions()
{
    bool success;
    bool allsuccess = true;
    
    for (int cnt = OrdersTotal() - 1; cnt >= 0; cnt --){
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber() == Magic && OrderSymbol() == Symbol()) {
         int i = 0;
         while(i < NumRetries) {
            i += 1;
            while(IsTradeContextBusy())Sleep(RetryDelayInSec*1000);
      	   RefreshRates();
            
            Print("Try "+i+" : Close "+OrderTicket());
            if(OrderType() == OP_BUY || OrderType() == OP_SELL)
               success = OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 99, 0);
            else
               success = OrderDelete(OrderTicket());
               
            if(!success){
               Print("Failed to close order "+OrderTicket()+" Error code:"+GetLastError());
               if(i == NumRetries) {
                  allsuccess = false;
                  Print("*** Final retry to CLOSE ORDER failed. Close trade manually ***");
               }
            } else {
               i = NumRetries;
            }
         }
      }
    }
    
    return(allsuccess); 
}

void PrintLN(string name, string text, double y = 0, int x = 0, string font = "Verdana") {
   if (ObjectFind(name) != 0) {
      ObjectCreate(name, OBJ_LABEL, 0, 0, 0);
      ObjectSet(name, OBJPROP_CORNER, 0);
   }
   
   ObjectSetText(name, text, FontSize, font, FontColor);
   ObjectSet(name, OBJPROP_XDISTANCE,  x);
   ObjectSet(name, OBJPROP_YDISTANCE,  y);
   ObjectSet(name, OBJPROP_CORNER, TableCorner);
}